package ca.recrown.islandsurvivalcraft.world;

import java.util.Random;

import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Biome;
import org.bukkit.generator.ChunkGenerator.BiomeGrid;
import org.bukkit.generator.ChunkGenerator.ChunkData;

import ca.recrown.islandsurvivalcraft.world.generation.BedrockGenerator;
import ca.recrown.islandsurvivalcraft.world.generation.IslandBiomeGenerator;
import ca.recrown.islandsurvivalcraft.world.shaders.WorldHeightShader;

/**
 * A world generator.
 */
public class IslandWorldGenerator {
    private final int maxHeight;
    private final BedrockGenerator bedrockGenerator;
    private final BiomeSelector biomeSelector;
    private final IslandBiomeGenerator biomeGenerator;
    private final WorldHeightShader heightShader;

    public final World world;
    public final IslandWorldBaseTerrainMap islandMapGenerator;
    public final Random random;
    
    public IslandWorldGenerator(World world, IslandBiomeGenerator islandBiomeGenerator, Random random) {
        this.world = world;
        this.maxHeight = world.getMaxHeight();
        this.random = random;
        this.biomeSelector = new BiomeSelector(random);
        this.bedrockGenerator = new BedrockGenerator(random);
        this.islandMapGenerator = new IslandWorldBaseTerrainMap(random, world.getSeaLevel(), 4);
        this.biomeGenerator = islandBiomeGenerator;
        this.heightShader = new WorldHeightShader(world.getSeed(), islandMapGenerator);
        biomeSelector.initialize();
        islandBiomeGenerator.initialize(world, islandMapGenerator, biomeSelector);
    }

    public void GenerateChunk(int chunkX, int chunkZ, int localX, int localZ, ChunkData chunk, BiomeGrid biomeGrid) {
        int worldX = 16 * chunkX + localX;
        int worldZ = 16 * chunkZ + localZ;
        //gets the bedrock.
        int bedrockHeight = bedrockGenerator.getBedrockHeight(worldX, worldZ);
        
        //Sets the biome.
        Biome currentBiome = biomeGenerator.GenerateBiome(chunkX, chunkZ, localX, localZ);
        for (int y = 0; y < maxHeight; y++) {
            biomeGrid.setBiome(localX, y, localZ, currentBiome);
        } 
        
        //get height shader.
        int height = heightShader.getAltitude(worldX, worldZ, currentBiome);

        
        //set general shape
        chunk.setRegion(localX, 0, localZ, localX, height, localZ, Material.BEDROCK);

        //set bedrock last
        chunk.setRegion(localX, 0, localZ, localX, bedrockHeight, localZ, Material.BEDROCK);
    }
}