package ca.recrown.islandsurvivalcraft.world.generation;

import org.bukkit.World;
import org.bukkit.block.Biome;

import ca.recrown.islandsurvivalcraft.pathfinding.CoordinateTargetValidatable;
import ca.recrown.islandsurvivalcraft.pathfinding.DepthFirstSearch;
import ca.recrown.islandsurvivalcraft.world.BiomeSelector;
import ca.recrown.islandsurvivalcraft.world.IslandWorldBaseTerrainMap;

public class BiomePerIslandGenerator implements IslandBiomeGenerator, CoordinateTargetValidatable {
    private boolean initialized;
    private IslandWorldBaseTerrainMap islandLocator;
    private BiomeSelector biomeSelector;
    private TemperatureMapGenerator temperatureMapGenerator;
    private World world;
    private DepthFirstSearch dfs;
    private Biome mainBiome;
    private Biome shoreBiome;
    private float temperature;

    public BiomePerIslandGenerator() {
        this.temperatureMapGenerator = new TemperatureMapGenerator();
    }
    
    @Override
    public void initialize(World world, IslandWorldBaseTerrainMap mapGenerator, BiomeSelector biomeSelector) {
        if (initialized) throw new IllegalStateException("Biome generator already initialized.");
        initialized = true;
        this.islandLocator = mapGenerator;
        this.biomeSelector = biomeSelector;
        this.world = world;
        this.temperatureMapGenerator.setSeed(world.getSeed());
        dfs = new DepthFirstSearch(islandLocator);
    }

    @Override
    public boolean isCoordinateTarget(int x, int y) {
        if (world.isChunkGenerated(x/16, y/16)) {
            Biome foundBiome = world.getBiome(x, 0, y);
            if (islandLocator.isShore(x, y)) {
                shoreBiome = foundBiome;
            } else {
                mainBiome = foundBiome;
            }
            if (mainBiome != null && shoreBiome != null) {
                return true;
            }
        }
        return false;
    }

    @Override
    public Biome GenerateBiome(int chunkX, int chunkZ, int localX, int localZ) {
        int worldX = chunkX * 16 + localX;
        int worldZ = chunkZ * 16 + localZ;

        if (islandLocator.isIsland(worldX, worldZ)) {
            if (mainBiome == null && shoreBiome == null) {
                dfs.setStartPosition(worldX, worldZ);
                if (!dfs.findTarget(this)) {
                    temperature = temperatureMapGenerator.getTemperature(worldX, worldZ);
                    mainBiome = biomeSelector.getLandBiome(temperature);
                    shoreBiome = biomeSelector.getShoreBiome(mainBiome, temperature);
                }
            }
        } else {
            mainBiome = null;
            shoreBiome = null;
        }

        
        Biome designatedBiome = mainBiome;
        if (mainBiome == null) {
            designatedBiome = biomeSelector.getOceanBiome(temperatureMapGenerator.getTemperature(worldX, worldZ));
        } else if (islandLocator.isShore(worldX, worldZ)) {
                designatedBiome = shoreBiome;
        }
        return designatedBiome;
    }

    @Override
    public IslandBiomeGenerator getInstance() {
        return new BiomePerIslandGenerator();
    }
}